/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File dialog.h - Header file containing the macros, data types and
 *		   prototypes for the SimCoupe dialog routines
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Display size (in SAM mode 4 pixels) */

#define DISPLAY_WIDTH	256
#define DISPLAY_HEIGHT	192

/* Special Key codes */

#define KEY_NONE	0
#define KEY_UP		1
#define KEY_DOWN	2
#define KEY_LEFT	3
#define KEY_RIGHT	4
#define KEY_CTRL	5
#define KEY_ENTER	6
#define KEY_ESCAPE	7
#define KEY_DELETE	8

/* Drawing Modes */

#define DM_OVER	0
#define DM_AND	1
#define DM_OR	2
#define DM_XOR	3

/* Shadow Modes */

#define SHADOW_OUT	0
#define SHADOW_IN	1

#define SH_WID		2

/* Clut offsets for special colours */

#define SHCOL_WHITE	0x0f
#define SHCOL_LIGHT	4
#define SHCOL_MEDIUM	2
#define SHCOL_DARK	1
#define SHCOL_BLACK	0

/* Clut offsets for masking */

#define MASK_ALL	0x0f
#define MASK_NONE	0x00

/* Character size constants */

#define GLYPH_WIDTH	6
#define GLYPH_HEIGHT	8

#define NO_GLYPHS	96

#define FONT_FILE	"font.bin"

/* Primitive types */

typedef struct _Pixmap {

	uns8  PixmapHeight;
	uns8  PixmapWidth;
	uns8 *PixmapData;

} Pixmap;


/* Pointer type */

#define BUTTON_1 0x01
#define BUTTON_2 0x02
#define BUTTON_3 0x04
#define BUTTON_4 0x08

typedef struct _Pointer {

	uns8 x;
	uns8 y;
	struct _ButtonState {
		int ButtonEvent;
		int ButtonStatus;
	} ButtonState;
	
	Pixmap *Glyph;
	Pixmap *Mask;

} Pointer;


/* Dialog Objects */

/* Object structure definitions */

#define MENU_OBJECT			0
#define FORM_OBJECT			1

#define MAX_MENU_ITEMS		25

#define MENU_DIALOG_ITEM	0x00
#define MENU_FUNCTION_ITEM	0x01
#define MENU_SUBMENU_ITEM	0x02
#define MENU_LINE_ITEM		0x04	

typedef struct _MenuObject {

	int NoMenuItems;
	int CurrentItem;
	struct _MenuItem {
	
		int   ItemType;
		char *ItemText;		
		void *ItemPtr;

	} MenuItems[MAX_MENU_ITEMS];	

} MenuObject;

#define FORM_ACTIVE	0x80

#define FORM_TEXT	0x01
#define FORM_ICON	0x02

#define FORM_DIALOG_BUTTON		0x81
#define FORM_FUNCTION_BUTTON	0x82
#define FORM_LIST_BOX			0X84
#define FORM_INPUT_BOX			0x88
#define FORM_SWITCH				0x90

/* Primitive types */



typedef struct _FormItem {

	int  ItemType;
	uns8 DeltaX, DeltaY;
	uns8 width, height;
	void *ItemPtr;	

	struct _FormItem *NextItem;
	struct _FormItem *LastItem;
	
} FormItem;

typedef struct _FormObject {

	int NoPassiveItems;
	FormItem *PassiveItemList;

	FormItem *CurrentActiveItem;
	FormItem *ActiveItemList;	
	int CurrentItemIsSelected;	
} FormObject;


/* The main Dialog Structure */

typedef struct _DialogObject {

/* These items are common to all dialog object types */

	int	 ObjectType;
	uns8	 xpos, ypos;
	uns8	 width, height;
	uns8	*BackingStore;
	struct  _DialogObject *ParentObject;

	int	 IsMapped;

/* The rest of the Object is a Union which depends on the different Object types */

	union _ObjectUnion {
	
		MenuObject	AMenu;
		FormObject	AForm;	

	} ObjectUnion;
} DialogObject;

/* Public function prototypes */


/* Dialog core code : from dialog_core.c */

void DialogInit(void);
void DialogCloseDown(void);
void DialogStart(void);
void DialogEnd(void);
void DialogCore(void);

DialogObject *CreateDialogObject(int TheType, uns8 x, uns8 y, uns8 TheWidth, uns8 TheHeight, 
		   DialogObject *TheParent);
void FreeDialogObject(DialogObject *TheDialog);
void MapDialog(DialogObject *TheDialog);
void UnMapDialog(DialogObject *TheDialog);
void DialogInitFont(void);
void DialogFreeFont(void);
void DialogSetActiveDialogObject(DialogObject *TheDialog);
DialogObject *DialogGetActiveDialogObject(void);
void DialogSetActiveDialogToParent(void);
void DialogSetActiveDialogToRoot(void);

/* Menu Object type : from menu.c */

DialogObject *MenuCreate(uns8 x, uns8 y, DialogObject *TheParent);
void MenuFree(DialogObject * TheDialog);
int  MenuAddItem(DialogObject *TheDialog, int TheType, char *TheText, void* TheItemPtr);
void MenuDraw(DialogObject *TheDialog);
void MenuHighLight(DialogObject *TheDialog, int DoHighLight);
void MenuSelectItem(DialogObject *TheDialog, int TheItem);
void MenuCore(DialogObject *TheDialog);


/* Form Object type : from form.c */

DialogObject *FormCreate(uns8 x, uns8 y, uns8 width, uns8 height, DialogObject *TheParent);
void FormFree(DialogObject * TheDialog);
FormItem *FormAddTextItem(DialogObject *TheDialog, uns8 DeltaX, uns8 DeltaY, char *TheText);
FormItem *FormAddSwitchItem(DialogObject *TheDialog, uns8 DeltaX, uns8 DeltaY, 
				  uns8 Height, uns8 Width, 
				  int *ValuePtr, int OnValue, int OffValue);
FormItem *FormAddButtonItem(DialogObject *TheDialog, uns8 DeltaX, uns8 DeltaY, char *TheText, int ButtonType, void* ActionPtr);
FormItem *FormAddListBoxItem(DialogObject *TheDialog, uns8 DeltaX, uns8 DeltaY, int NoItems, 
		        uns8 TextWidth, uns8 TextHeight, 
		        char **TheList, void (*CBFunction)(char *));
FormItem *FormAddInputBoxItem(DialogObject *TheDialog, uns8 DeltaX, uns8 DeltaY, int NoChars, 
			uns8 TextWidth, 
			char *InputDefault, void (*CBFunction)(char *));
void FormUpdateListBoxItem(DialogObject *TheDialog, FormItem *TheFormItem, 
			   int NoItems, char **TheList);
void FormUpdateInputBoxItem(DialogObject *TheDialog, FormItem *TheFormItem, 
		      int ScrollOffset, int NoChars, char *InputDefault);
void FormDrawTextItem(FormItem *TheFormItem, uns8 FormXPos, uns8 FormYPos);
void FormDrawButtonItem(FormItem *TheFormItem, uns8 FormXPos, uns8 FormYPos);
void FormDrawSwitchItem(FormItem *TheFormItem, uns8 FormXPos, uns8 FormYPos);
void FormDrawListBoxItem(FormItem *TheFormItem, uns8 FormXPos, uns8 FormYPos);
void FormDrawInputBoxItem(FormItem *TheFormItem, uns8 FormXPos, uns8 FormYPos);

void FormDrawItem(FormItem *TheFormItem, uns8 FormXPos, uns8 FormYPos);
void FormDraw(DialogObject *TheDialog);
void FormHighLight(DialogObject *TheDialog, FormItem *TheFormItem, int DoHighLight);
void FormCore(DialogObject *TheDialog);

/* File selector Object : from file_selector.c */

void FileSelectorOpen(char *ThePath, char *TheFilter, void (*CBfunction)(char *),
		      DialogObject *ParentDialog);

/* Keyboard functions : from keyboard.c */

int GetSpecialKey(void);
char GetAsciiKey(void);

/* Pointer functions : from pointer.c */

void PointerInit( void );
void PointerEnter( void );
void PointerExit( void );
void PointerClose( void );
void PointerSetStatus( int TheDeltaX, int TheDeltaY, int TheButtonState );
void PointerSetGlyph( Pixmap *TheGlyph, Pixmap *TheMask );
Pointer *PointerGetStatus( void );
uns8 PointerGetXPosition( void );
uns8 PointerGetYPosition( void );
int  PointerGetButtonState( int *TheStatus );
int PointerInsideRegion( uns8 TheXpos, uns8 TheYpos, uns8 TheWidth, uns8 TheHeight );
void PointerDraw( void );
void PointerErase( void );


/* Graphics primitives : from primitives.c */

void DrawLineHorz(uns8 x, uns8 y, uns8 Length, uns8 Colour, int DrawMode);
void DrawLineVert(uns8 x, uns8 y, uns8 Length, uns8 Colour, int DrawMode);
void FillArea(uns8, uns8, uns8 width, uns8 height, uns8 Colour, int DrawMode);
void DrawString(uns8 x, uns8 y, char *TheString, uns8 Colour, int DrawMode);
void DrawShadowBox(uns8 x, uns8 y, uns8 width, uns8 height, int ShadowMode, int FillBox, int DrawMode);
void DrawBox(uns8 x, uns8 y, uns8 width, uns8 height, int Colour, int DrawMode);
void ClearDisplay(void);
void FetchArea(uns8 *Buffer, uns8 x, uns8 y, uns8 width, uns8 height);
void StoreArea(uns8 *Buffer, uns8 x, uns8 y, uns8 width, uns8 height);
void DrawPixmap(uns8 x, uns8 y, Pixmap *ThePixmap, int DrawMode);

/* external variables definitions */

extern uns8 		*DialogFontPtr;
extern DialogObject	*ActiveDialogObject;		/* The Active Dialog */
