/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File file_selector.c : User interface for SimCoupe : File selector dialog
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* The FileSelector dialog is just a specialised use of the 'form' dialog,   */
/* which allows the user to browse through the file system and select a file.*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <fnmatch.h>

#include "z80.h"
#include "mem.h"
#include "sam.h"
#include "util.h"

#include "dialog.h"

/* Private types */

typedef struct _FileList {
	int NoFiles;
	char **FileNames;
} FileList;


/* Private function prototypes */

FileList *FileSelectorGetDirList(char *ThePath); 
FileList *FileSelectorGetFileList(char *ThePath, char *TheFilter);
void FileSelectorFreeFileList(FileList *TheFileList);

void CBFileSelectorOK(void);
void CBFileSelectorFilter(void);
void CBFileSelectorCancel(void);
void CBFileSelectorSelectDir(char *TheDir);
void CBFileSelectorSelectFile(char *TheFile);
void CBFileSelectorSelectFilePath(char *ThePath);
void CBFileSelectorSelectFilter(char *TheFilter);

/* Internal File Selector global variables */

#define MAX_PATH	1024
#define MAX_DIRECTORIES	1024
#define MAX_FILES	1024

static DialogObject *TheFileSelectorDialog = NULL;
static FormItem *DirListBox;
static FormItem *FileListBox;
static FormItem *PathInputBox;

static char CurrentPath[MAX_PATH+1];
static char CurrentFilter[255];

static char SelectedFile[MAX_PATH];

static FileList *CurrentDirectories;
static FileList *CurrentFiles;


void (*CBuser)(char *);


void
FileSelectorOpen(char *ThePath, char *TheFilter, void (*CBfunction)(char *),
		 DialogObject *ParentDialog)
{
	static int FirstTime = True;

	/* The core File Selector function : Create the form dialog and */
	/* initialise the internal selector variables 			*/

	if (TheFileSelectorDialog != NULL)
		FormFree(TheFileSelectorDialog); 

	TheFileSelectorDialog = FormCreate(10, 5, 245, 184, ParentDialog);

	/* First time in copy the default file path and filter */
	if (FirstTime) {
		FirstTime = !FirstTime;
		strcpy(CurrentPath,   ThePath);
		strcpy(CurrentFilter, TheFilter);
	}

	FormAddTextItem(TheFileSelectorDialog, 10, 24, "Directories");
	FormAddTextItem(TheFileSelectorDialog, 130, 24, "Files");

	/* Initialise the List boxes */

	if ((CurrentDirectories = FileSelectorGetDirList(ThePath)) != NULL) {
		DirListBox = FormAddListBoxItem(TheFileSelectorDialog, 10, 34, 
				   CurrentDirectories->NoFiles, 16, 12, 
				   CurrentDirectories->FileNames, CBFileSelectorSelectDir);
	}
	if ((CurrentFiles = FileSelectorGetFileList(ThePath, TheFilter)) != NULL) {
		FileListBox = FormAddListBoxItem(TheFileSelectorDialog, 130, 34, 
				   CurrentFiles->NoFiles, 16, 12, 
				   CurrentFiles->FileNames, CBFileSelectorSelectFile);
	}

	FormAddTextItem(TheFileSelectorDialog, 10, 10, "Filter");
	FormAddInputBoxItem(TheFileSelectorDialog, 48, 7, sizeof(CurrentFilter), 30, 
			CurrentFilter, CBFileSelectorSelectFilter);

	FormAddTextItem(TheFileSelectorDialog, 10, 146, "File");
	PathInputBox = FormAddInputBoxItem(TheFileSelectorDialog, 48, 143, 
				       sizeof(CurrentPath), 30, 
				       CurrentPath, CBFileSelectorSelectFilePath);

	FormAddButtonItem(TheFileSelectorDialog, 10, 164, "  OK  ", FORM_FUNCTION_BUTTON, CBFileSelectorOK);
/*	FormAddButtonItem(TheFileSelectorDialog, 100, 164, "Filter", FORM_FUNCTION_BUTTON, NULL);*/
	FormAddButtonItem(TheFileSelectorDialog, 188, 164, "Cancel", FORM_FUNCTION_BUTTON, CBFileSelectorCancel);


	SelectedFile[0] = '\0';
	CBuser = CBfunction;

	DialogSetActiveDialogObject(TheFileSelectorDialog);
}

void
FileSelectorClose(void)
{
	/* Free up the memory used by the file selector */

	FileSelectorFreeFileList(CurrentDirectories);
	FileSelectorFreeFileList(CurrentFiles);
	FormFree(TheFileSelectorDialog);
	TheFileSelectorDialog = NULL;
}

FileList *
FileSelectorGetDirList(char *ThePath) 
{
	/* Search ThePath and return all the directories  */

	DIR *TheDirPtr;
	struct dirent *TheDirEntry;
	struct stat    stbuff;	
	char FileBuffer[1024];

	static FileList AFileList;
	/* This array really should be dynamically allocated */
	static char *DirectoriesStore[MAX_DIRECTORIES];


	int NoDirs = 0;

	/* First Open the Directory */
	
	if ((TheDirPtr = opendir(ThePath)) != NULL) {
	
		/* Read all the file entries */
		while ((TheDirEntry = readdir(TheDirPtr)) != NULL && NoDirs < MAX_DIRECTORIES) {
			/* complete the path and test if the file is a directory */
#ifndef __DJGPP
			sprintf(FileBuffer,"%s/%s", ThePath, TheDirEntry->d_name);
#else
			sprintf(FileBuffer,"%s\\%s", ThePath, TheDirEntry->d_name);
#endif
			if (!stat(FileBuffer, &stbuff)) {
				if ((stbuff.st_mode & S_IFMT) == S_IFDIR &&
				    (stbuff.st_mode & S_IRUSR) && 
				    strcmp(TheDirEntry->d_name, ".") != 0 
				    ) {
					/* Directory */
					DirectoriesStore[NoDirs] = (char *)malloc(1+strlen(TheDirEntry->d_name));
					strcpy(DirectoriesStore[NoDirs++], TheDirEntry->d_name);
				}
			}	
				
		}
		closedir(TheDirPtr);

		AFileList.NoFiles   = NoDirs;
		AFileList.FileNames = DirectoriesStore;
		return(&AFileList);
	}
	else {
		return(NULL);
	}
}

FileList *
FileSelectorGetFileList(char *ThePath, char *TheFilter) 
{
	/* Search ThePath and return all the files which match TheFilter */

	DIR *TheDirPtr;
	struct dirent *TheDirEntry;
	struct stat    stbuff;	
	char FileBuffer[1024];
	/* This array really should be dynamically allocated */
	static char *FilesStore[MAX_FILES];

	static FileList AFileList;
	int NoFiles = 0;

	/* First Open the Directory */
	
	if ((TheDirPtr = opendir(ThePath)) != NULL) {
	
		/* Read all the file entries */
		while ((TheDirEntry = readdir(TheDirPtr)) != NULL && NoFiles < MAX_FILES) {
			/* complete the path and test if the file is a directory */
#ifndef __DJGPP
			sprintf(FileBuffer,"%s/%s", ThePath, TheDirEntry->d_name);
#else
			sprintf(FileBuffer,"%s\\%s", ThePath, TheDirEntry->d_name);
#endif
			if (!stat(FileBuffer, &stbuff)) {
				if ((stbuff.st_mode & S_IFMT) != S_IFDIR &&
				    (stbuff.st_mode & S_IRUSR)) {
					/* File */
					if (!fnmatch(CurrentFilter, FileBuffer, 0)) {
						/* Matched Filter Pattern */
						FilesStore[NoFiles] = (char *)malloc(1+strlen(TheDirEntry->d_name));
						strcpy(FilesStore[NoFiles++], TheDirEntry->d_name);
					}
				}
			}	
				
		}
		closedir(TheDirPtr);

		AFileList.NoFiles   = NoFiles;
		AFileList.FileNames = FilesStore;
		return(&AFileList);
	}
	else {
		return(NULL);
	}

}


void
FileSelectorFreeFileList(FileList *TheFileList)
			  
{
	/* Free up the storage allocated to the (Directory/File)List */
	int i;
	
	for (i=0; i < TheFileList->NoFiles; i++)
		free(TheFileList->FileNames[i]);
}


/* FileSelector Internal Call Backs */

void
CBFileSelectorCancel(void)
{
	/* Call Back for the OK button : Close the Selector dialog and */
	/* exit after unmapping the dialog		       				   */

	UnMapDialog(TheFileSelectorDialog);
	DialogSetActiveDialogToParent(); 
	FileSelectorClose();

}

void
CBFileSelectorOK(void)
{
	/* Call Back for the Cancel button : Close the Selector dialog and */
	/* exit via the user Call back function CBuser		       */

	UnMapDialog(TheFileSelectorDialog);
	DialogSetActiveDialogToParent(); 
	FileSelectorClose();
	if (CBuser != NULL)
		CBuser(SelectedFile);
}



void
CBFileSelectorSelectDir(char *TheDir)
{
	/* Call Back for the Directory list box : change the working */
	/* directory and update the list boxes			     */

	if (strlen(CurrentPath)+strlen(TheDir) < MAX_PATH) {
		/* Prevent occurences of 'x/../y' in the path */
		if (strcmp(TheDir, "..") !=0 ) {
#ifndef __DJGPP
			/* The root path ("/") is a special case */
			if (strcmp(CurrentPath, "/") != 0)
				sprintf(CurrentPath, "%s/%s",CurrentPath, TheDir);
			else
				sprintf(CurrentPath, "/%s", TheDir);
#else
			/* The root path ("x:\") is a special case */
			if (strcmp(CurrentPath+1, ":\\") != 0)
				sprintf(CurrentPath, "%s\\%s",CurrentPath, TheDir);
			else
				sprintf(CurrentPath+1, ":\\%s", TheDir);
#endif
		}
		else {
#ifndef __DJGPP
			char *LastSeparator;
			/* Check there isn't a '/' at the end of the path */
			if (CurrentPath[strlen(CurrentPath)-1] == '/')
				CurrentPath[strlen(CurrentPath)-1] = '\0';
			LastSeparator = strrchr(CurrentPath, '/');
			if (LastSeparator != NULL && (LastSeparator-CurrentPath)) {
				/* Check there isn't a '/' at the end of the path */
				if (*(LastSeparator+1) == '\0') {
					*LastSeparator = '\0';
					LastSeparator = strrchr(CurrentPath, '/');
				}
				CurrentPath[LastSeparator - CurrentPath] = '\0';
			}
			else
				CurrentPath[1] = '\0';
#else

			char *LastSeparator;
			/* Check there isn't a '\' at the end of the path */
			if (CurrentPath[strlen(CurrentPath)-1] == '\\' &&
			    strlen(CurrentPath) > 3)
				CurrentPath[strlen(CurrentPath)-1] = '\0';
			LastSeparator = strrchr(CurrentPath, '\\');
			if (LastSeparator != NULL && (LastSeparator-CurrentPath-2)) {
				CurrentPath[LastSeparator - CurrentPath] = '\0';
			}
			else
				CurrentPath[3] = '\0';
#endif
		}
		
		if ((CurrentDirectories = FileSelectorGetDirList(CurrentPath)) != NULL &&
		    (CurrentFiles = FileSelectorGetFileList(CurrentPath, CurrentFilter)) != NULL) {
		}
		FormUpdateListBoxItem(TheFileSelectorDialog, DirListBox, 
					CurrentDirectories->NoFiles,
					CurrentDirectories->FileNames);
		FormUpdateListBoxItem(TheFileSelectorDialog, FileListBox,
					CurrentFiles->NoFiles,
					CurrentFiles->FileNames);
		FormUpdateInputBoxItem(TheFileSelectorDialog, PathInputBox,
				       0, strlen(CurrentPath), CurrentPath);
	}	

}

void
CBFileSelectorSelectFile(char *TheFile)
{

	if (strlen(CurrentPath)+strlen(CurrentPath) < MAX_PATH) {
#ifndef __DJGPP
		sprintf(SelectedFile, "%s/%s", CurrentPath, TheFile);
#else
		sprintf(SelectedFile, "%s\\%s", CurrentPath, TheFile);
#endif

		FormUpdateListBoxItem(TheFileSelectorDialog, FileListBox,
					CurrentFiles->NoFiles,
					CurrentFiles->FileNames);
		FormUpdateInputBoxItem(TheFileSelectorDialog, PathInputBox,
				       0, strlen(CurrentPath), SelectedFile);

	}

}


void
CBFileSelectorSelectFilter(char *TheFilter)
{
	strcpy(CurrentFilter, TheFilter);
	
	if ((CurrentDirectories = FileSelectorGetDirList(CurrentPath)) != NULL &&
	    (CurrentFiles = FileSelectorGetFileList(CurrentPath, CurrentFilter)) != NULL) {
	}
	FormUpdateListBoxItem(TheFileSelectorDialog, DirListBox, 
						CurrentDirectories->NoFiles,
						CurrentDirectories->FileNames);
	FormUpdateListBoxItem(TheFileSelectorDialog, FileListBox,
						CurrentFiles->NoFiles,
						CurrentFiles->FileNames);

}

void
CBFileSelectorSelectFilePath(char *ThePath)
{
	/* Call Back function for Pathname input box */ 

	struct stat stbuff;	

	/* The action depends on whether the path is a file or a directory */

	if (!stat(ThePath, &stbuff)) {
		if ((stbuff.st_mode & S_IFMT) == S_IFDIR &&
		    (stbuff.st_mode & S_IRUSR) ) {
			/* Directory */
			strcpy(CurrentPath, ThePath);
			if ((CurrentDirectories = FileSelectorGetDirList(CurrentPath)) != NULL &&
			    (CurrentFiles = FileSelectorGetFileList(CurrentPath, CurrentFilter)) != NULL) {
			}
			FormUpdateListBoxItem(TheFileSelectorDialog, DirListBox, 
						CurrentDirectories->NoFiles,
						CurrentDirectories->FileNames);
			FormUpdateListBoxItem(TheFileSelectorDialog, FileListBox,
						CurrentFiles->NoFiles,
						CurrentFiles->FileNames);
			FormUpdateInputBoxItem(TheFileSelectorDialog, PathInputBox,
					       0, strlen(CurrentPath), CurrentPath);
		}
		else if ((stbuff.st_mode & S_IFMT) != S_IFDIR &&
			 (stbuff.st_mode & S_IRUSR) ) {
			/* File */	
			strcpy(SelectedFile, ThePath);
			CBFileSelectorOK();
		}
	}
}
