
/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File main.c - initial setup etc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define Z80_MAIN

#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

#include "z80.h"
#include "sam.h"
#include "mem.h"
#include "resource.h"
#include "util.h"
#include "fdi.h"
#include "interface.h"

static void got_sig(int signo);
static void read_roms(void);

void timer_stats(void);

int
main(int argc, char **argv)
{
	/* Make sure we can't be killed off while initialising stuff, 
	 * or we might end up leaving shared memory segments lying about.  */
	signal(SIGINT,  SIG_IGN);
	signal(SIGHUP,  SIG_IGN);
	signal(SIGTERM, SIG_IGN);
	signal(SIGQUIT, SIG_IGN);

	/* Initailise the emulator resources */
	
	get_resources(&argc, argv);
	
	ui_init();

	init_display();

	init_memory();
	init_ports();
	init_pages();
	read_roms();

	eject_disk(1);
	eject_disk(2);
        
	insert_disk(GetCfg(disk_1), 1);
	insert_disk(GetCfg(disk_2), 2);

	signal(SIGINT,  got_sig);
	signal(SIGHUP,  got_sig);
	signal(SIGTERM, got_sig);
	signal(SIGQUIT, got_sig);

	/* Insert UI close down function into QUIT sequence */
	OnQuit(ui_end);

	/* Insert timing stats function into Quit sequence */
	OnQuit(timer_stats);

	/* Start timing the emulator */
	startwatch(0);

	emulator_main_loop();

	return 0;
}


/* Read the ROM image <image> into real memory page <page> */
static int
load_rom_image(char *image, int page)
{
	FILE *ifp;
	size_t nread;

	if ((ifp = open_simcoupe_file(image, "r")) == NULL) {
		simcoupe_mesg(SIMCOUPE_ERR, "can't open ROM image <%s>", image);
		return -1;
	}

	nread = fread(realMem[page], 1, SAM_PAGESIZE, ifp);

	/* give a warning if image is not 16K */
	if (nread != SAM_PAGESIZE ) {
		simcoupe_mesg(SIMCOUPE_WARN, "ROM image <%s> is only %d bytes", image, nread);
	}

	fclose(ifp);

	return 0;
}


void
read_roms(void)
{
		load_rom_image(GetCfg(rom0), ROM0);
		load_rom_image(GetCfg(rom1), ROM1);
}


static void
got_sig(int signo)
{
	Quit(0);
}

void 
timer_stats(void)
{
	extern long n_frames;
	long elapsed = stopwatch();
	double samtime = n_frames/50;

	simcoupe_mesg(SIMCOUPE_INFO,"Sam time : %.2f",samtime);
	simcoupe_mesg(SIMCOUPE_INFO,"Real time: %lu.%03lu",elapsed/1000,elapsed%1000);
	simcoupe_mesg(SIMCOUPE_INFO,"Ratio    : %.3f",samtime/elapsed*1000.);

}