/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File vgaspectrum.c - alternative version of the keypress/release 
 *                      code for when the SAM is in spectrum mode
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <vgakeyboard.h>

#include "z80.h"
#include "vgaspectrum.h"
#include "sam.h"
#include "resource.h"
#include "util.h"

/* Extra scancodes not included in vgakeyboard.h */

#define SCANCODE_UK_HASH	43
#define SCANCODE_UK_BACKSLASH	86


/* Maps vgalib scancodes to a Sam keyboard row and bit to set/reset */
/* E.g. ktbl[SCANCODE_A] means set/reset bit 0 (0xfe) of keyboard row 1 */
static struct ktbl {
	int port;	/* which keyboard row is the key on? */
	int bit;	/* which bit to mask */
} keytable[51] = {
	{9, 0}, {9, 0}, 
	/* 0 1 2 3 4 5 6 7 8 9 */
        {3, 0xfe}, {3, 0xfd}, {3, 0xfb}, {3, 0xf7}, {3, 0xef}, 
        {4, 0xef}, {4, 0xf7}, {4, 0xfb}, {4, 0xfd}, {4, 0xfe},
	
	{9, 0}, {9, 0}, {9, 0}, {9,0},

	/* Q W E R T Y U I O P */
	{2, 0xfe}, {2, 0xfd}, {2, 0xfb}, {2, 0xf7}, {2, 0xef},
	{5, 0xef}, {5, 0xf7}, {5, 0xfb}, {5, 0xfd}, {5, 0xfe},
	
	{9, 0}, {9, 0}, {9, 0}, {9,0},

	/* A S D F G H J K L */
	{1, 0xfe}, {1, 0xfd}, {1, 0xfb}, {1, 0xf7}, {1, 0xef},
	{6, 0xef}, {6, 0xf7}, {6, 0xfb}, {6, 0xfd},
	
	{9, 0}, {9, 0}, {9, 0}, {9,0}, {9,0},

	/* Z X C V B N M */
	           {0, 0xfd}, {0, 0xfb}, {0, 0xf7}, {0, 0xef},
	{7, 0xef}, {7, 0xf7}, {7, 0xfb}
	           	
};

/* from vgasam.c */
extern int SpectrumKeyboard;
extern uns8 keyports[]; 
extern int *key_table;
extern uns8 inPorts[256], outPorts[256];

/* from processor.c */
extern int disass;
extern int InterruptType;


static int ShiftPressed = False;	/* Flag for shift key */
static int LastKeyScancode = 0xff;		/* Use to stop auto repeat */

/* Process a KeyPress event */
void
process_spectrum_keypress(int ks)
{
	int i;	

	if (ks == LastKeyScancode)
		return;

	/* First check for the 'special keys' */
	if (ks == key_table[KM_JOY_UP])
		inPorts[0x1f] |= 0x02;
	else if (ks == key_table[KM_JOY_DOWN])
		inPorts[0x1f] |= 0x04;
	else if (ks == key_table[KM_JOY_LEFT])
		inPorts[0x1f] |= 0x10;
	else if (ks == key_table[KM_JOY_RIGHT])
		inPorts[0x1f] |= 0x08;
	else if (ks == key_table[KM_JOY_FIRE])
		inPorts[0x1f] |= 0x01;

	switch (ks) {
		/* Emulator control keys */
		case SCANCODE_F1: 
			InterruptType = Z80_reset;
			for (i = 0; i < 9; i++)
				keyports[i] = 0xff;
			break;
		case SCANCODE_F2:	/* Switch Keyboard mode */
			SpectrumKeyboard = !SpectrumKeyboard;
			break;
		case SCANCODE_F10: case SCANCODE_F12:
			Quit(0);
			break;

		/* For the shift key we need to set a flag */
		case SCANCODE_LEFTSHIFT: case SCANCODE_RIGHTSHIFT:
			keyports[0] &= 0xfe; 
			ShiftPressed = True;
			break;


		/* Ordinary Spectrum keys */
		case SCANCODE_SPACE:
			keyports[7] &= 0xfe; break;
		case SCANCODE_TAB:
			keyports[3] &= 0xbf; break;
		case SCANCODE_ENTER:
			keyports[6] &= 0xfe; break;
		case SCANCODE_LEFTCONTROL: case SCANCODE_RIGHTCONTROL:
		/* <Shift> <Control> is mapped to Caps Lock */
			if (!(keyports[0] & 0x01))
			     keyports[3] &= 0xfd;
			break;
		case SCANCODE_LEFTALT:
/*              case SCANCODE_LEFTMETA: ????? */
			keyports[7] &= 0xfd; break;

		case SCANCODE_RIGHTALT: 
/*              case SCANCODE_RIGHTMETA: case SCANCODE_Mode_switch: ????? */
			keyports[0] &= 0xfe; 
			keyports[3] &= 0xfe;
			break;
		case SCANCODE_BACKSPACE: 
/*              case SCANCODE_DELETE: ????? */
			keyports[0] &= 0xfe; 
			keyports[4] &= 0xfe;
			break;
		case SCANCODE_ESCAPE:
			keyports[3] &= 0xdf; break;
		case SCANCODE_CURSORBLOCKUP:
			keyports[0] &= 0xfe;
			keyports[4] &= 0xf7; break;
		case SCANCODE_CURSORBLOCKDOWN:
			keyports[0] &= 0xfe;
			keyports[4] &= 0xef; break;
		case SCANCODE_CURSORBLOCKLEFT:
			keyports[0] &= 0xfe;
			keyports[3] &= 0xef; break;
		case SCANCODE_CURSORBLOCKRIGHT:
			keyports[0] &= 0xfe;
			keyports[4] &= 0xfb; break;
		default:
			if (ks < 52) {
				keyports[keytable[ks].port] &= keytable[ks].bit;
			}
			break;
	}
}

/* Process a KeyRelease event */
void
process_spectrum_keyrelease(int ks)
{

	/* First check for the 'special keys' */
	if (ks == key_table[KM_JOY_UP])
		inPorts[0x1f] &= ~0x02;
	else if (ks == key_table[KM_JOY_DOWN])
		inPorts[0x1f] &= ~0x04;
	else if (ks == key_table[KM_JOY_LEFT])
		inPorts[0x1f] &= ~0x10;
	else if (ks == key_table[KM_JOY_RIGHT])
		inPorts[0x1f] &= ~0x08;
	else if (ks == key_table[KM_JOY_FIRE])
		inPorts[0x1f] &= ~0x01;

	switch (ks) {

		/* For the shift key we need to reset a flag */
		case SCANCODE_LEFTSHIFT: case SCANCODE_RIGHTSHIFT:
			keyports[0] |= ~0xfe; 
			ShiftPressed = False;
			break;

		/* Ordinary Spectrum keys */
		case SCANCODE_SPACE:
			keyports[7] &= 0xfe; break;
		case SCANCODE_TAB:
			keyports[3] &= 0xbf; break;
		case SCANCODE_ENTER:
			keyports[6] &= 0xfe; break;
		case SCANCODE_LEFTCONTROL: case SCANCODE_RIGHTCONTROL:
		/* <Shift> <Control> is mapped to Caps Lock */
			if (!(keyports[0] & 0x01))
			     keyports[3] &= 0xfd;
			break;
		case SCANCODE_LEFTALT:
/*              case SCANCODE_LEFTMETA: ????? */
			keyports[7] &= 0xfd; break;

		case SCANCODE_RIGHTALT: 
/*              case SCANCODE_RIGHTMETA: case SCANCODE_Mode_switch: ????? */
			keyports[0] &= 0xfe; 
			keyports[3] &= 0xfe;
			break;
		case SCANCODE_BACKSPACE: 
/*              case SCANCODE_DELETE: ????? */
			keyports[0] &= 0xfe; 
			keyports[4] &= 0xfe;
			break;
		case SCANCODE_ESCAPE:
			keyports[3] &= 0xdf; break;
		case SCANCODE_CURSORBLOCKUP:
			keyports[0] &= 0xfe;
			keyports[4] &= 0xf7; break;
		case SCANCODE_CURSORBLOCKDOWN:
			keyports[0] &= 0xfe;
			keyports[4] &= 0xef; break;
		case SCANCODE_CURSORBLOCKLEFT:
			keyports[0] &= 0xfe;
			keyports[3] &= 0xef; break;
		case SCANCODE_CURSORBLOCKRIGHT:
			keyports[0] &= 0xfe;
			keyports[4] &= 0xfb; break;
		default:
			if (ks < 52) {
				keyports[keytable[ks].port] &= keytable[ks].bit;
			}
			break;
	}
}

/* Process a KeyRelease event */
void
process_spectrum_keyrelease(int ks)
{
	int i;	

	if (ks == LastKeyScancode)
		LastKeyScancode = 0xff;

	/* First check for the 'special keys' */
	if (ks == key_table[KM_JOY_UP])
		inPorts[0x1f] &= ~0x02;
	else if (ks == key_table[KM_JOY_DOWN])
		inPorts[0x1f] &= ~0x04;
	else if (ks == key_table[KM_JOY_LEFT])
		inPorts[0x1f] &= ~0x10;
	else if (ks == key_table[KM_JOY_RIGHT])
		inPorts[0x1f] &= ~0x08;
	else if (ks == key_table[KM_JOY_FIRE])
		inPorts[0x1f] &= ~0x01;

	switch (ks) {

		/* For the shift key we need to reset a flag */
		case SCANCODE_LEFTSHIFT: case SCANCODE_RIGHTSHIFT:
			keyports[0] |= ~0xfe; 
			ShiftPressed = False;
			break;

		/* Ordinary Spectrum keys */
		case SCANCODE_SPACE:
			keyports[7] |= ~0xfe; break;
		case SCANCODE_TAB:
			keyports[3] |= ~0xbf; break;
		case SCANCODE_ENTER:
			keyports[6] |= ~0xfe; break;
		case SCANCODE_LEFTCONTROL: case SCANCODE_RIGHTCONTROL:
		/* <Shift> <Control> is mapped to Caps Lock */
			     keyports[3] |= ~0xfd;
		case SCANCODE_LEFTALT:
/*              case SCANCODE_LEFTMETA: ????? */
			keyports[7] |= ~0xfd; break;

		case SCANCODE_RIGHTALT: 
/*              case SCANCODE_RIGHTMETA: case SCANCODE_Mode_switch: ????? */
			keyports[0] |= ~0xfe;
			keyports[3] |= ~0xfe; break;
		case SCANCODE_BACKSPACE: 
/*              case SCANCODE_DELETE: ????? */
			keyports[0] |= ~0xfe; 
			keyports[4] |= ~0xfe;
			break;
		case SCANCODE_ESCAPE:
			keyports[3] |= ~0xdf; break;
		case SCANCODE_CURSORBLOCKUP:
			keyports[0] |= ~0xfe;
			keyports[4] |= ~0xf7; break;
		case SCANCODE_CURSORBLOCKDOWN:
			keyports[0] |= ~0xfe;
			keyports[4] |= ~0xef; break;
		case SCANCODE_CURSORBLOCKLEFT:
			keyports[0] |= ~0xfe;
			keyports[3] |= ~0xef; break;
		case SCANCODE_CURSORBLOCKRIGHT:
			keyports[0] |= ~0xfe;
			keyports[4] |= ~0xfb; break;
		default:
			if (ks < 52) {
				keyports[keytable[ks].port] |= ~keytable[ks].bit;
			}
			break;
	}
}
